# -*- coding: utf-8; -*-

from setuptools import setup

setup(name='emojipy',
      version='2.9.9',
      description='Python wrapper for emojione',
      author='Kaviraj Kanagaraj',
      author_email='kaviraj@launchyard.com',
      packages=['emojipy',],
      test_suite='emojipy.tests',
      url = 'https://github.com/phuong/emojipy', 
      download_url = 'https://github.com/phuong/emojipy/archive/master.zip', 
      keywords = ['emojipy', 'emojione', 'python emoji'], 
)
